@@ -1,6 +1,6 @@
 import requests
 import json
-
+import logging
 
 class AuthException(Exception):
     pass
@@ -73,10 +73,11 @@ class ClientBase(object):
                  api_version,
                  endpoints,
                  server_name,
+                 verify=True
                  ):
         self._server_address = server_address
         self._default_url_mapping = {server_name: self._server_address}
-
+        self.verify=verify
         self.session = requests.Session()
         head_val = auth_header.get('Authorization', None)
         if head_val is not None:
@@ -126,7 +127,9 @@ class ClientBase(object):
 
         if http_error_msg:
             raise requests.HTTPError(http_error_msg, response=r)
-
+        warning = r.headers.get('Warning')
+        if warning:
+            logging.warning(warning)
 
 class ClientBaseWithDataset(ClientBase):
     def __init__(self,
@@ -135,7 +138,8 @@ class ClientBaseWithDataset(ClientBase):
                  api_version,
                  endpoints,
                  server_name,
-                 dataset_name
+                 dataset_name,
+                 verify=True
                  ):
 
         super(ClientBaseWithDataset, self).__init__(server_address,
@@ -143,9 +147,11 @@ class ClientBaseWithDataset(ClientBase):
                                                     api_version,
                                                     endpoints,
                                                     server_name,
+                                                    verify=verify
                                                     )
         self._dataset_name = dataset_name
 
+
     @property
     def dataset_name(self):
         return self._dataset_name
