@@ -24,6 +24,9 @@ class CGEncoder(json.JSONEncoder):
 
 
 def package_bounds(bounds):
+    if (bounds.shape != (3,2)):
+        raise ValueError('Bounds must be a 3x2 matrix (x,y,z) x (min,max) in chunkedgraph resolution voxel units')
+    
     bounds_str = []
     for b in bounds:
         bounds_str.append("-".join(str(b2) for b2 in b))
@@ -100,7 +103,7 @@ class ChunkedGraphClientV1(ClientBase):
         else:
             return timestamp
 
-    def get_roots(self, supervoxel_ids, timestamp=None):
+    def get_roots(self, supervoxel_ids, timestamp=None, stop_level=None):
         """Get the root id for a specified supervoxel
 
         Parameters
@@ -109,12 +112,17 @@ class ChunkedGraphClientV1(ClientBase):
             Supervoxel ids values
         timestamp : datetime.datetime, optional
             UTC datetime to specify the state of the chunkedgraph at which to query, by default None. If None, uses the current time.
+        stop_level : int or None, optional
+            If True, looks up ids only up to a given stop level. Default is None.
 
         Returns
         -------
         np.array(np.uint64)
             Root IDs containing each supervoxel.
         """
+        if stop_level is not None:
+            return self._get_level_roots(supervoxel_ids, stop_level=stop_level, timestamp=timestamp)
+
         endpoint_mapping = self.default_url_mapping
         url = self._endpoints['get_roots'].format_map(endpoint_mapping)
         query_d = package_timestamp(self._process_timestamp(timestamp))
@@ -151,6 +159,31 @@ class ChunkedGraphClientV1(ClientBase):
         handle_response(response, as_json=False)
         return np.int64(response.json()['root_id'])
 
+    def _get_level_roots(self, supervoxel_ids, stop_level, timestamp=None):
+        """Get the root id for a specified supervoxel
+
+        Parameters
+        ----------
+        supervoxel_id : np.uint64
+            Supervoxel id value
+        timestamp : datetime.datetime, optional
+            UTC datetime to specify the state of the chunkedgraph at which to query, by default None. If None, uses the current time.
+
+        Returns
+        -------
+        np.uint64
+            Root ID containing the supervoxel.
+        """
+        endpoint_mapping = self.default_url_mapping
+
+        url = self._endpoints['handle_roots'].format_map(endpoint_mapping)
+        query_d = package_timestamp(self._process_timestamp(timestamp))
+        query_d['stop_layer'] = stop_level
+        data = {'node_ids': supervoxel_ids}
+        response = self.session.post(
+            url, data=json.dumps(data, cls=CGEncoder), params=query_d)
+        return handle_response(response, as_json=True).get('root_ids', None)
+
     def get_merge_log(self, root_id):
         """Get the merge log (splits and merges) for an object
 
@@ -319,6 +352,24 @@ class ChunkedGraphClientV1(ClientBase):
         l2_path = np.array(resp_d['l2_path'])
 
         return centroids, l2_path, failed_l2_ids
+    
+    def get_subgraph(self, root_id, bounds):
+        """Get subgraph of root id within a bounding box
+
+        Args:
+            root_id ([int64]): root (or seg_id/node_id) of chunkedgraph to query
+            bounds ([np.array]): 3x2 bounding box (x,y,z)x (min,max) in chunkedgraph coordinates
+        """
+        endpoint_mapping = self.default_url_mapping
+        endpoint_mapping['root_id'] = root_id
+        url = self._endpoints['get_subgraph'].format_map(endpoint_mapping)
+        query_d = {}
+        if bounds is not None:
+            query_d['bounds'] = package_bounds(bounds)
+        
+        response = self.session.get(url, params=query_d)
+        rd = handle_response(response)
+        return np.int64(rd['nodes']), np.double(rd['affinities']), np.int32(rd['areas'])
 
     def level2_chunk_graph(self, root_id):
         """Get graph of level 2 chunks, the smallest agglomeration level above supervoxels.
@@ -441,7 +492,7 @@ class ChunkedGraphClientLegacy(ClientBase):
             if self._default_timestamp is not None:
                 timestamp = self._default_timestamp
             else:
-                timestamp = datetime.utcnow()
+                timestamp = datetime.datetime.utcnow()
 
         endpoint_mapping = self.default_url_mapping
         url = self._endpoints['get_roots'].format_map(endpoint_mapping)
@@ -481,7 +532,7 @@ class ChunkedGraphClientLegacy(ClientBase):
             if self._default_timestamp is not None:
                 timestamp = self._default_timestamp
             else:
-                timestamp = datetime.utcnow()
+                timestamp = datetime.datetime.utcnow()
 
         endpoint_mapping = self.default_url_mapping
         endpoint_mapping['supervoxel_id'] = supervoxel_id
